﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using Styx;
using Styx.Helpers;
using Styx.CommonBot.POI;
using Styx.CommonBot;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.WoWInternals;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;
using System.Xml.Serialization;

namespace MailboxManagerFree
{
    public partial class MailboxManager
    {
        private BotPoi CurrentPOI { get { return BotPoi.Current; } }

        public static string SavePath
        {
            get { return string.Format("{0}\\MailboxManager\\{1}-{2}\\RecipientList.xml", Settings.SettingsDirectory, StyxWoW.Me.Name, Lua.GetReturnVal<string>("return GetRealmName()", 0)); }
        }

        
        public static void LoadSettings()
        {
            if (!File.Exists(SavePath))
                return;
            
            XmlSerializer deserializer = new XmlSerializer(typeof(BindingList<Item>));
            TextReader reader = new StreamReader(SavePath);
            BindingList<Item> list;
            list = (BindingList<Item>)deserializer.Deserialize(reader);
            MailboxManager.items = list;
            reader.Close();
            LogS("Settings Loaded Successfully.");
        }

        public static void ImportSettings()
        {
            LogS("Import Settings Dialog Opened.");
            OpenFileDialog popup = new OpenFileDialog();
            popup.Filter = "Xml Files | *.xml";
            popup.DefaultExt = "xml";
            if (popup.ShowDialog() == DialogResult.OK)
            {
                if (!File.Exists(popup.FileName))
                    return;

                XmlSerializer deserializer = new XmlSerializer(typeof(BindingList<Item>));
                TextReader reader = new StreamReader(popup.FileName);
                BindingList<Item> list;
                list = (BindingList<Item>)deserializer.Deserialize(reader);
                MailboxManager.items = list;
                reader.Close();

                LogS("Settings Imported Successfully.");
            }
            else
                LogS("Settings Import Cancelled.");
        }

        public static void SaveSettings()
        {
            if (!Directory.Exists(Path.GetDirectoryName(SavePath)))
                Directory.CreateDirectory(Path.GetDirectoryName(SavePath));

            XmlSerializer serializer = new XmlSerializer(typeof(BindingList<Item>));
            TextWriter textWriter = new StreamWriter(SavePath);
            serializer.Serialize(textWriter, MailboxManager.items);
            textWriter.Close();
            LogS("Settings Saved Successfully.");

            
        }

        public static void ExportSettings()
        {
            LogS("Settings Export Dialog Opened.");
            SaveFileDialog popup = new SaveFileDialog();
            popup.Filter = "Xml Files | *.xml";
            popup.DefaultExt = "xml";
            if (popup.ShowDialog() == DialogResult.OK)
            {
                if (!Directory.Exists(Path.GetDirectoryName(popup.FileName)))
                    Directory.CreateDirectory(Path.GetDirectoryName(popup.FileName));

                XmlSerializer serializer = new XmlSerializer(typeof(BindingList<Item>));
                TextWriter textWriter = new StreamWriter(popup.FileName);
                serializer.Serialize(textWriter, MailboxManager.items);
                textWriter.Close();

                LogS("Settings Exported Sucessfully.");
            }
            else
                LogS("Settings Export Cancelled.");
        }

        public static void LogS(string str)
        {
            Logging.Write(string.Format("[{0}] {1}",MailboxManager.name,str));
        }

    }

    public class Money : IComparable<Money>
    {
        /// <summary>
        /// Amount of gold.
        /// </summary>
        public int Gold;
        /// <summary>
        /// Amount of silver.
        /// </summary>
        public int Silver;
        /// <summary>
        /// Amount of copper.
        /// </summary>
        public int Copper;
        /// <summary>
        /// Gets or sets the total amount of copper.
        /// </summary>
        /// <value>
        /// The total amount of copper.
        /// </value>
        public int TotalCopper
        {
            get { return Gold * 10000 + Silver * 100 + Copper; }
            set
            {
                Gold = (int)Math.Floor((double)Math.Abs(value / 10000));
                Silver = (int)Math.Floor((double)Math.Abs((value / 100) % 100));
                Copper = (int)Math.Floor((double)Math.Abs(value % 100));
            }
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="Money" /> class.
        /// </summary>
        /// <param name="totalCopper">The total amount of copper.</param>
        public Money(int totalCopper)
        {
            TotalCopper = totalCopper;
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="Money" /> class.
        /// </summary>
        /// <param name="gold">The amount of gold.</param>
        /// <param name="silver">The amount of silver.</param>
        /// <param name="copper">The amount ofcopper.</param>
        public Money(int gold, int silver, int copper)
        {
            Gold = gold;
            Silver = silver;
            Copper = copper;
        }
        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance. Example: 2g10s5c or 5s0c if there's 0 gold.
        /// </summary>
        /// 
        public Money()
        {
            Gold = 0;
            Silver = 0;
            Copper = 0;
        }
        /// 
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance. Example: 2g10s5c or 5s0c if there's 0 gold.
        /// </returns>
        public override string ToString()
        {
            var moneyString = new StringBuilder();
            if (Gold > 0)
            {
                moneyString.Append(Gold);
                moneyString.Append("g");
                moneyString.Append(Silver);
                moneyString.Append("s");
                moneyString.Append(Copper);
                moneyString.Append("c");
            }
            else if (Silver > 0)
            {
                moneyString.Append(Silver);
                moneyString.Append("s");
                moneyString.Append(Copper);
                moneyString.Append("c");
            }
            else
            {
                moneyString.Append(Copper);
                moneyString.Append("c");
            }
            return moneyString.ToString();
        }
        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <param name="copper">The total amount of copper.</param>
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance.
        /// </returns>
        public static string ToString(int copper)
        {
            var money = new Money(copper);
            return money.ToString();
        }
        public override bool Equals(object obj)
        {
            return GetHashCode() == obj.GetHashCode();
        }
        public override int GetHashCode()
        {
            return TotalCopper;
        }
        public static Money Zero
        {
            get
            {
                return new Money(0);
            }
        }
        public int CompareTo(Money other)
        {
            if (TotalCopper < other.TotalCopper) return 1;
            if (TotalCopper > other.TotalCopper) return -1;
            return 0;
        }
        // +
        public static Money operator +(Money m1, Money m2)
        {
            return new Money(m1.TotalCopper + m2.TotalCopper);
        }
        public static Money operator +(Money m1, int number)
        {
            return m1 + new Money(number);
        }
        public static Money operator +(int number, Money m1)
        {
            return m1 + number;
        }
        public static Money operator +(Money m1, float number)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return m1 + integer;
        }
        public static Money operator +(float number, Money m1)
        {
            return m1 + number;
        }
        public static Money operator +(Money m1, double number)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return m1 + integer;
        }
        public static Money operator +(double number, Money m1)
        {
            return m1 + number;
        }
        public static Money operator +(Money m1, decimal number)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return m1 + integer;
        }
        public static Money operator +(decimal number, Money m1)
        {
            return m1 + number;
        }

        // -
        public static Money operator -(Money m1, Money m2)
        {
            return new Money(m1.TotalCopper - m2.TotalCopper);
        }
        public static Money operator -(Money m1, int number)
        {
            return m1 - new Money(number);
        }
        public static Money operator -(int number, Money m1)
        {
            return new Money(number) - m1;
        }
        public static Money operator -(Money m1, float number)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return m1 - integer;
        }
        public static Money operator -(float number, Money m1)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return integer - m1;
        }
        public static Money operator -(Money m1, double number)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return m1 - integer;
        }
        public static Money operator -(double number, Money m1)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return integer - m1;
        }
        public static Money operator -(Money m1, decimal number)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return m1 - integer;
        }
        public static Money operator -(decimal number, Money m1)
        {
            var integer = Convert.ToInt32(Math.Round(number));
            return integer - m1;
        }

        // ><
        public static bool operator >(Money m1, Money m2)
        {
            return m1.TotalCopper > m2.TotalCopper;
        }
        public static bool operator <(Money m1, Money m2)
        {
            return m1.TotalCopper < m2.TotalCopper;
        }

        public static bool operator >(Money m1, int number)
        {
            return m1.TotalCopper > number;
        }
        public static bool operator <(Money m1, int number)
        {
            return m1.TotalCopper < number;
        }

        public static bool operator >(int number, Money m1)
        {
            return number > m1.TotalCopper;
        }
        public static bool operator <(int number, Money m1)
        {
            return number < m1.TotalCopper;
        }

        public static bool operator >(Money m1, float number)
        {
            return m1.TotalCopper > number;
        }
        public static bool operator <(Money m1, float number)
        {
            return m1.TotalCopper < number;
        }

        public static bool operator >(float number, Money m1)
        {
            return number > m1.TotalCopper;
        }
        public static bool operator <(float number, Money m1)
        {
            return number < m1.TotalCopper;
        }

        public static bool operator >(Money m1, double number)
        {
            return m1.TotalCopper > number;
        }
        public static bool operator <(Money m1, double number)
        {
            return m1.TotalCopper < number;
        }

        public static bool operator >(double number, Money m1)
        {
            return number > m1.TotalCopper;
        }
        public static bool operator <(double number, Money m1)
        {
            return number < m1.TotalCopper;
        }

        public static bool operator >(Money m1, decimal number)
        {
            return m1.TotalCopper > number;
        }
        public static bool operator <(Money m1, decimal number)
        {
            return m1.TotalCopper < number;
        }

        public static bool operator >(decimal number, Money m1)
        {
            return number > m1.TotalCopper;
        }
        public static bool operator <(decimal number, Money m1)
        {
            return number < m1.TotalCopper;
        }

        // >= <=
        public static bool operator >=(Money m1, Money m2)
        {
            return m1.TotalCopper >= m2.TotalCopper;
        }
        public static bool operator <=(Money m1, Money m2)
        {
            return m1.TotalCopper <= m2.TotalCopper;
        }

        public static bool operator >=(Money m1, int number)
        {
            return m1.TotalCopper >= number;
        }
        public static bool operator <=(Money m1, int number)
        {
            return m1.TotalCopper <= number;
        }

        public static bool operator >=(int number, Money m1)
        {
            return number >= m1.TotalCopper;
        }
        public static bool operator <=(int number, Money m1)
        {
            return number <= m1.TotalCopper;
        }

        public static bool operator >=(Money m1, float number)
        {
            return m1.TotalCopper >= number;
        }
        public static bool operator <=(Money m1, float number)
        {
            return m1.TotalCopper <= number;
        }

        public static bool operator >=(float number, Money m1)
        {
            return number >= m1.TotalCopper;
        }
        public static bool operator <=(float number, Money m1)
        {
            return number <= m1.TotalCopper;
        }

        public static bool operator >=(Money m1, double number)
        {
            return m1.TotalCopper >= number;
        }
        public static bool operator <=(Money m1, double number)
        {
            return m1.TotalCopper <= number;
        }

        public static bool operator >=(double number, Money m1)
        {
            return number >= m1.TotalCopper;
        }
        public static bool operator <=(double number, Money m1)
        {
            return number <= m1.TotalCopper;
        }

        public static bool operator >=(Money m1, decimal number)
        {
            return m1.TotalCopper >= number;
        }
        public static bool operator <=(Money m1, decimal number)
        {
            return m1.TotalCopper <= number;
        }

        public static bool operator >=(decimal number, Money m1)
        {
            return number >= m1.TotalCopper;
        }
        public static bool operator <=(decimal number, Money m1)
        {
            return number <= m1.TotalCopper;
        }

        // *
        public static Money operator *(Money m1, Money m2)
        {
            return new Money(m1.TotalCopper * m2.TotalCopper);
        }
        public static Money operator *(Money m1, int number)
        {
            return new Money(m1.TotalCopper * number);
        }
        public static Money operator *(int number, Money m1)
        {
            return new Money(number * m1.TotalCopper);
        }
        public static Money operator *(Money m1, float number)
        {
            int newValue = Convert.ToInt32(Math.Floor(m1.TotalCopper * number));
            return new Money(newValue);
        }
        public static Money operator *(float number, Money m1)
        {
            return m1 * number;
        }
        public static Money operator *(Money m1, double number)
        {
            int newValue = Convert.ToInt32(Math.Floor(m1.TotalCopper * number));
            return new Money(newValue);
        }
        public static Money operator *(double number, Money m1)
        {
            return m1 * number;
        }
        public static Money operator *(Money m1, decimal number)
        {
            int newValue = Convert.ToInt32(Math.Floor(m1.TotalCopper * number));
            return new Money(newValue);
        }
        public static Money operator *(decimal number, Money m1)
        {
            return m1 * number;
        }

        // /
        public static Money operator /(Money m1, Money m2)
        {
            return new Money(m1.TotalCopper / m2.TotalCopper);
        }
        public static Money operator /(Money m1, int number)
        {
            return new Money(m1.TotalCopper / number);
        }
        public static Money operator /(int number, Money m1)
        {
            return new Money(number / m1.TotalCopper);
        }
        public static Money operator /(Money m1, float number)
        {
            int newValue = Convert.ToInt32(Math.Floor(m1.TotalCopper / number));
            return new Money(newValue);
        }
        public static Money operator /(float number, Money m1)
        {
            int newValue = Convert.ToInt32(Math.Floor(number / m1.TotalCopper));
            return new Money(newValue);
        }
        public static Money operator /(Money m1, double number)
        {
            int newValue = Convert.ToInt32(Math.Floor(m1.TotalCopper * number));
            return new Money(newValue);
        }
        public static Money operator /(double number, Money m1)
        {
            int newValue = Convert.ToInt32(Math.Floor(number / m1.TotalCopper));
            return new Money(newValue);
        }
        public static Money operator /(Money m1, decimal number)
        {
            int newValue = Convert.ToInt32(Math.Floor(m1.TotalCopper / number));
            return new Money(newValue);
        }
        public static Money operator /(decimal number, Money m1)
        {
            int newValue = Convert.ToInt32(Math.Floor(number / m1.TotalCopper));
            return new Money(newValue);
        }

        public static bool operator ==(Money m1, Money m2)
        {
            Debug.Assert(m1 != null, "m1 != null");
            Debug.Assert(m2 != null, "m2 != null");
            return m1.TotalCopper == m2.TotalCopper;
        }

        public static bool operator !=(Money m1, Money m2)
        {
            Debug.Assert(m1 != null, "m1 != null");
            Debug.Assert(m2 != null, "m2 != null");
            return m1.TotalCopper != m2.TotalCopper;
        }
        public static bool operator ==(Money m1, int number)
        {
            Debug.Assert(m1 != null, "m1 != null");
            return m1.TotalCopper == number;
        }
        public static bool operator !=(Money m1, int number)
        {
            Debug.Assert(m1 != null, "m1 != null");
            return m1.TotalCopper != number;
        }
        public static bool operator ==(int number, Money m1)
        {
            return m1 == number;
        }
        public static bool operator !=(int number, Money m1)
        {
            return m1 != number;
        }
    }

    public static class Extensions
    {
        public static Money ToMoney(this string str)
        {
            int gold = 0;
            int silver = 0;
            int copper = 0;
            if (str != null)
            {
                try
                {
                    var moneyString = str.ToLowerInvariant();
                    var match = Regex.Match(moneyString, "([0-9]+(g|s|c))");
                    if (match.Success)
                    {
                        var goldRegex = Regex.Match(moneyString, "([0-9]+g)");
                        if (goldRegex.Success)
                        {
                            gold = int.Parse(goldRegex.Value.Replace("g", ""));
                        }
                        var silverRegex = Regex.Match(moneyString, "([0-9]+s)");
                        if (silverRegex.Success)
                        {
                            silver = int.Parse(silverRegex.Value.Replace("s", ""));
                        }
                        var copperRegex = Regex.Match(moneyString, "([0-9]+c)");
                        if (copperRegex.Success)
                        {
                            copper = int.Parse(copperRegex.Value.Replace("c", ""));
                        }
                    }
                    else
                    {
                        int.TryParse(str, out copper);
                    }
                    var r = copper + (silver * 100) + (gold * 10000);
                    return new Money(r);
                }
                catch (Exception)
                {
                    //throw;
                }
            }
            return new Money();
        }

        public static String ToHiddenName(this String s)
        {
            string j = "";
            int u = s.Count() - 3;
            for (int x = 0; x < u; x++)
                j += "*";
            return string.Format(s.ToCharArray().First()  + (j) + s.ToCharArray().Last());
            
        }

        public static XElement ToXml(this string EleName, string str)
        {
            return new XElement(str, EleName);
        }

        public static XElement ToXml(this int EleDbl, string str)
        {
            return new XElement(str, EleDbl.ToString());
        }

        public static XElement ToXml(this KeyValuePair<int,int> pair, string str)
        {
            return new XElement(str, 
                new XElement("Item", pair.Key.ToString()),
                new XElement("Price", pair.Value.ToString()));
        }

        public static XElement ToXml(this KeyValuePair<string, int> pair, string str)
        {
            return new XElement(str,
                new XElement("Item", pair.Key),
                new XElement("Price", pair.Value.ToString()));
        }

    }

}
